@@ -1,3 +1,6 @@
+#[doc(hidden)]
+pub extern crate swc_ecma_ast;
+
 pub use self::{
     factory::ExprFactory,
     ident::{id, Id},
@@ -182,41 +185,49 @@ pub trait StmtLike: Sized + 'static {
 impl ModuleItemLike for Stmt {}
 
 impl StmtLike for Stmt {
+    #[inline]
     fn try_into_stmt(self) -> Result<Stmt, Self> {
         Ok(self)
     }
+    #[inline]
     fn as_stmt(&self) -> Option<&Stmt> {
         Some(&self)
     }
+    #[inline]
     fn from_stmt(stmt: Stmt) -> Self {
         stmt
     }
 }
 
 impl ModuleItemLike for ModuleItem {
+    #[inline]
     fn try_into_module_decl(self) -> Result<ModuleDecl, Self> {
         match self {
             ModuleItem::ModuleDecl(decl) => Ok(decl),
             _ => Err(self),
         }
     }
+    #[inline]
     fn try_from_module_decl(decl: ModuleDecl) -> Result<Self, ModuleDecl> {
         Ok(ModuleItem::ModuleDecl(decl))
     }
 }
 impl StmtLike for ModuleItem {
+    #[inline]
     fn try_into_stmt(self) -> Result<Stmt, Self> {
         match self {
             ModuleItem::Stmt(stmt) => Ok(stmt),
             _ => Err(self),
         }
     }
+    #[inline]
     fn as_stmt(&self) -> Option<&Stmt> {
         match *self {
             ModuleItem::Stmt(ref stmt) => Some(stmt),
             _ => None,
         }
     }
+    #[inline]
     fn from_stmt(stmt: Stmt) -> Self {
         ModuleItem::Stmt(stmt)
     }
@@ -2073,7 +2084,7 @@ impl VisitMut for IdentReplacer<'_> {
 
 pub struct BindingCollector<I>
 where
-    I: IdentLike + Eq + Hash,
+    I: IdentLike + Eq + Hash + Send + Sync,
 {
     only: Option<SyntaxContext>,
     bindings: AHashSet<I>,
@@ -2082,7 +2093,7 @@ where
 
 impl<I> BindingCollector<I>
 where
-    I: IdentLike + Eq + Hash,
+    I: IdentLike + Eq + Hash + Send + Sync,
 {
     fn add(&mut self, i: &Ident) {
         if let Some(only) = self.only {
@@ -2097,10 +2108,38 @@ where
 
 impl<I> Visit for BindingCollector<I>
 where
-    I: IdentLike + Eq + Hash,
+    I: IdentLike + Eq + Hash + Send + Sync,
 {
     noop_visit_type!();
 
+    fn visit_module_items(&mut self, nodes: &[ModuleItem], _: &dyn Node) {
+        #[cfg(feature = "concurrent")]
+        if nodes.len() > 128 {
+            use rayon::prelude::*;
+            let set = nodes
+                .par_iter()
+                .map(|node| {
+                    let mut v = BindingCollector {
+                        only: self.only,
+                        bindings: Default::default(),
+                        is_pat_decl: self.is_pat_decl,
+                    };
+                    node.visit_with(&Invalid { span: DUMMY_SP }, &mut v);
+                    v.bindings
+                })
+                .reduce(AHashSet::default, |mut a, b| {
+                    a.extend(b);
+                    a
+                });
+            self.bindings.extend(set);
+            return;
+        }
+
+        for node in nodes {
+            node.visit_children_with(self)
+        }
+    }
+
     fn visit_class_decl(&mut self, node: &ClassDecl, _: &dyn Node) {
         node.visit_children_with(self);
 
@@ -2171,7 +2210,7 @@ where
 /// Collects binding identifiers.
 pub fn collect_decls<I, N>(n: &N) -> AHashSet<I>
 where
-    I: IdentLike + Eq + Hash,
+    I: IdentLike + Eq + Hash + Send + Sync,
     N: VisitWith<BindingCollector<I>>,
 {
     let mut v = BindingCollector {
@@ -2187,7 +2226,7 @@ where
 /// identical to `ctxt`.
 pub fn collect_decls_with_ctxt<I, N>(n: &N, ctxt: SyntaxContext) -> AHashSet<I>
 where
-    I: IdentLike + Eq + Hash,
+    I: IdentLike + Eq + Hash + Send + Sync,
     N: VisitWith<BindingCollector<I>>,
 {
     let mut v = BindingCollector {
